package ca.canadapost.cpcdp.returns;

import ca.canadapost.cpcdp.returns.generated.messages.Messages;
import ca.canadapost.cpcdp.returns.generated.authreturn.*;
import ca.canadapost.cpcdp.returns.generated.authreturn.ParcelCharacteristicsType.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

import java.math.*;

/**
 * Sample code for the CreateAuthorizedReturn Canada Post service.
 * <p>
 * The CreateAuthorizedReturn service is used to create a return shipping label 
 * when the sender address and approximate package weight are known. This type of 
 * label is typically emailed through a self-serve or service agent process to the
 * individual returning the item.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 * Jersey REST client using JAXB for CreateAuthReturn Request.  <br>
 *  USAGE:<pre>
 *        CreateAuthReturnClient client = new CreateAuthReturnClient(username, password);
 *        ClientResponse response = client.createAuthReturn(return);
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class CreateAuthReturnClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Returns_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = mailedBy;    	
    	
    	// REST URL
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/authorizedreturn";
    	
		// Create CreateAuthReturn XML Request Object
        AuthorizedReturn myReturn = new AuthorizedReturn();
              
		DomesticAddressDetailsType returnerAddress = new DomesticAddressDetailsType();
		returnerAddress.setAddressLine1("502 MAIN ST N");
		returnerAddress.setCity("MONTREAL");
		returnerAddress.setProvince("QC");
		returnerAddress.setPostalCode("H2B1A0");
		
		ReturnerType returner = new ReturnerType();
		returner.setName("Bulma");
		returner.setCompany("Capsule Corp.");
		returner.setDomesticAddress(returnerAddress);
		
		DomesticAddressDetailsType receiverAddress = new DomesticAddressDetailsType();
		receiverAddress.setAddressLine1("123 Postal Drive");
		receiverAddress.setCity("Ottawa");
		receiverAddress.setProvince("ON");
		receiverAddress.setPostalCode("K1P5Z9");
		
		ReceiverType receiver = new ReceiverType();
		receiver.setName("John Doe");
		receiver.setCompany("ACME Corp.");
		receiver.setDomesticAddress(receiverAddress);
		
		Dimensions parcelDimension = new Dimensions();
		parcelDimension.setHeight(new BigDecimal(6));
		parcelDimension.setLength(new BigDecimal(12));
		parcelDimension.setWidth(new BigDecimal(9));

		ParcelCharacteristicsType characteristics = new ParcelCharacteristicsType();
		characteristics.setWeight(new BigDecimal(5));		
		characteristics.setDimensions(parcelDimension);
		
		PrintPreferencesType printPreferences = new PrintPreferencesType();
		printPreferences.setEncoding("PDF");
		printPreferences.setOutputFormat("8.5x11");
		
		AuthSettlementInfoType settlementInfo = new AuthSettlementInfoType();
		settlementInfo.setContractId("0042708517");
						
		myReturn.setServiceCode("DOM.EP");
		myReturn.setReceiver(receiver);
		myReturn.setReturner(returner);
		myReturn.setParcelCharacteristics(characteristics);
		myReturn.setPrintPreferences(printPreferences);
		myReturn.setSettlementInfo(settlementInfo);
    	
		// Execute CreateAuthReturn Request
        CreateAuthReturnClient myClient = new CreateAuthReturnClient(username, password);
        ClientResponse resp = myClient.createAuthReturn(myReturn);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(AuthorizedReturnInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches CreateAuthReturnInfo schema.
            if (entity instanceof AuthorizedReturnInfo) {
            	AuthorizedReturnInfo returnInfo = (AuthorizedReturnInfo) entity;
            	System.out.println("Tracking Pin: " + returnInfo.getTrackingPin());                
                for (Iterator<Link> iter = returnInfo.getLinks().getLinks().iterator(); iter.hasNext();) { 
                	Link aItem = (Link) iter.next();                	
	                System.out.println( aItem.getRel() + ": " + aItem.getHref());
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public CreateAuthReturnClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse createAuthReturn(Object xml) throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.authreturn-v2+xml").header("Content-Type", "application/vnd.cpc.authreturn-v2+xml").acceptLanguage("en-CA").post(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
