package ca.canadapost.cpcdp.returns;

import ca.canadapost.cpcdp.returns.generated.messages.Messages;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the DeleteOpenReturnTemplate Canada Post service.
 * <p>
 * The DeleteOpenReturnTemplate service is used to delete an existing active open
 * return template. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 * Jersey REST client using JAXB for DeleteOpenReturnTemplate Request.  <br>
 *  USAGE:<pre>
 *        DeleteOpenReturnTemplateClient client = new DeleteOpenReturnTemplateClient(username, password);
 *        ClientResponse response = client.deleteOpenReturnTemplate();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class DeleteOpenReturnTemplateClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {

    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Returns_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
    	
    	// REST URL
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/openreturn/349641323786705649";    	
    	
		// Execute DeleteOpenReturnTemplate Request
        DeleteOpenReturnTemplateClient myClient = new DeleteOpenReturnTemplateClient(username, password);
        ClientResponse resp = myClient.deleteOpenReturnTemplate();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        try {
            if ( resp.getStatus() == 204 ) {
            	// If server returns HTTP status code 204 then delete is successful.
                System.out.println("Template successfully deleted.");
            } else {
            	// Example of using JAXB to parse xml error response
                JAXBContext jc;            	
            	jc = JAXBContext.newInstance(Messages.class);
                Object entity = jc.createUnmarshaller().unmarshal(respIS);
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public DeleteOpenReturnTemplateClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse deleteOpenReturnTemplate() throws UniformInterfaceException {
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.delete(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
