package ca.canadapost.cpcdp.returns;

import ca.canadapost.cpcdp.returns.generated.messages.Messages;
import ca.canadapost.cpcdp.returns.generated.openreturn.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetOpenReturnTemplate Canada Post service.
 * <p>
 * The GetOpenReturnTemplate service is used to retrieve the information that 
 * allows access to the Retrieve Next Open Return Artifact service associated with
 * the template.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 * Jersey REST client using JAXB for GetOpenReturnTemplate Request.  <br>
 *  USAGE:<pre>
 *        GetOpenReturnTemplate client = new GetOpenReturnTemplate(username, password);
 *        ClientResponse response = client.getOpenReturnTemplate();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetOpenReturnTemplateClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Returns_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
		
    	// Use link from CreateOpenReturnTemplate
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/openreturn/349641323786705649";		
		
		// Execute GetOpenReturnTemplate Request
        GetOpenReturnTemplateClient myClient = new GetOpenReturnTemplateClient(username, password);
        ClientResponse resp = myClient.getOpenReturnTemplate();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(OpenReturnInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches OpenReturnInfo schema.
            if (entity instanceof OpenReturnInfo) {
            	OpenReturnInfo returnInfo = (OpenReturnInfo) entity;
            	System.out.println("Artifacts Remaining: " + returnInfo.getArtifactsRemaining());                
                for (Iterator<Link> iter = returnInfo.getLinks().getLinks().iterator(); iter.hasNext();) { 
                	Link aItem = (Link) iter.next();                	
	                System.out.println( aItem.getRel() + ": " + aItem.getHref());
                }              
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetOpenReturnTemplateClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getOpenReturnTemplate() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.openreturn-v2+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
